import { EventEmitter } from 'events';

interface Events<T> {
  /** When an element is set (added or overwritten) */
  on(event: 'set', listener: (element: T, index: number) => void): this;
  once(event: 'set', listener: (element: T, index: number) => void): this;
  emit(event: 'set', element: T, index: number): boolean;
  /** When an element is removed (explicitly removed - overwritten does not trigger this) */
  on(event: 'remove', listener: (element: T, index: number) => void): this;
  once(event: 'remove', listener: (element: T, index: number) => void): this;
  emit(event: 'remove', element: T, index: number): boolean;
}

/** A dictionary-like class that emits events */
export class EventedDictionary<T> extends EventEmitter implements Events<T> {
  private data: { [key: string]: T } = {};
  private count: number = 0;

  /**
   * Check if a given key is in use
   * @param key Key to check if used
   */
  public exists(key: string): boolean {
    return Object.prototype.hasOwnProperty.call(this.data, key);
  }
  
  /**
   * Get the element of a given key
   * @param key Key of the element
   */
  public get(key: string): T {
    return this.data[key];
  }

  /**
   * Set the element of a given key (adds the element)
   * @param key Desired key of the element
   * @param element Element to set
   */
  public set(key: string, element: T): T {
    // Increment count if key was not already in use
    if (!Object.prototype.hasOwnProperty.call(this.data, key)) { this.count++; }
    this.data[key] = element; // Set the element of the key
    this.emit('set', element, key); // Emit event
    return element; // Success
  }
  
  /**
   * Remove an element
   * @param key Key of the element
   */
  public remove(key: string): boolean {
    // Check if key is not present in data object
    if (!Object.prototype.hasOwnProperty.call(this.data, key)) { return false; }
    this.count -= 1; // Decrement count
    var element: T = this.data[key]; // Get element
    delete this.data[key]; // Remove element / key pair from data
    this.emit('remove', element, key); // Emit event
    return true; // Success
  }
  
  /**
   * Get the number of key/element pairs
   */
  public getCount(): number {
    return this.count;
  }
  
  /**
   * Call a function for each key/element pair
   * @param callback callback(element, key)
   */
  public forEach(callback: (element: T, key: string) => void): void {
    for (var key in this.data) {
      callback(this.data[key], key);
    }
  }
}
