import { EventEmitter } from 'events';

interface Events<T> {
  /** When an element is added */
  on(event: 'add', listener: (element: T, index: number) => void): this;
  once(event: 'add', listener: (element: T, index: number) => void): this;
  emit(event: 'add', element: T, index: number): boolean;
  /** When an element is removed */
  on(event: 'remove', listener: (element: T, index: number) => void): this;
  once(event: 'remove', listener: (element: T, index: number) => void): this;
  emit(event: 'remove', element: T, index: number): boolean;
}

/** An array-like class that emits events */
export class EventedList<T> extends EventEmitter implements Events<T> {
  private data: Array<T> = new Array<T>();

  /**
   * Get the element of a given index (or undefined if not found)
   * @param index Elements index
   */
  public get(index: number): T {
    return this.data[index|0];
  }

  /**
   * Add an element to the end of the list
   * @param element Element to add
   */
  public push(element: T): T {
    this.data.push(element); // Push element onto the array
    this.emit('add', element, this.data.length-1); // Emit event
    return element;
  }

  /**
   * Remove an element from the list
   * (Only the first occurrence is removed - lowest index first)
   * @param element Element to remove
   * @returns {boolean} If an element was removed
   */
  public remove(element: T): boolean {
    var index: number = this.data.indexOf(element); // Get index of element
    if (index === -1) { return false; } // Check if element was not found
    this.data.splice(index, 1); // Remove element from array
    this.emit('remove', element, index); // Emit event
    return true; // Success
  }

  /**
   * Remove an element with a given index
   * @param index Index of element to remove
   * @returns {boolean} If an element was removed
   */
  public removeAt(index: number): boolean {
    if (index < 0 || index > this.getLength()) { return false; } // Check if index is out of bounds
    var element: T = this.data[index]; // Get element (so it can later be sent in the event)
    this.data.splice(index, 1); // Remove element from array
    this.emit('remove', element, index); // Emit event
    return true; // Success
  }

  /**
   * Remove all elements from the list
   */
  public clear(): void {
    var length: number = this.data.length;
    for (var i = 0; i < length; i++) {
      this.removeAt(0);
    }
  }

  /**
   * Get the number of elements in the list
   * @returns {number} Number of elements in list
   */
  public getLength(): number {
    return this.data.length;
  }

  /**
   * Copy the underlying array
   */
  public copyData(): Array<T> {
    return this.data.slice(0);
  }

  /**
   * Copy this evented list (Copy has no event listeners)
   * @returns {EventedList<T>} Copy of this
   */
  public copy(): EventedList<T> {
    var list = new EventedList<T>();
    list.data = this.copyData();
    return list;
  }

  // -- Wrapped Array Methods -- //

  public forEach(func: (value: T, index: number)=>void): void {
    return this.data.forEach(func);
  }
  public indexOf(searchElement: T, fromIndex?: number): number {
    return this.data.indexOf(searchElement, fromIndex);
  }
  public some(func: (value: T, index: number)=>boolean, thisArg?: any): boolean {
    return this.data.some(func, thisArg);
  }
}
