/**
 * 2D Vector
 * @class Point
 * @property {Number} x 
 * @property {Number} y 
 */
export class Point implements IPoint {
  public x: number = 0;
  public y: number = 0;

  /**
   * Constructor
   * @param {Number} x X value (optional)
   * @param {Number} y Y value (optional)
   */
  constructor(x: number = 0, y: number = 0) {
    this.x = x;
    this.y = y;
  }

  /**
   * Calculates the distance to the origin.
   * @return {Number} Distance to origin
   */
  public length(): number {
    return Math.sqrt(this.x * this.x + this.y * this.y);
  }

  /**
   * Set the X and Y of this point.
   * @param {Number} x X
   * @param {Number} y Y
   * @return {Point} this
   */
  public set(x: number, y: number): Point {
    this.x = x;
    this.y = y;
    return this;
  }

  /**
   * Moves the point by some amount.
   * @param {Number} x X movement
   * @param {Number} y Y movement
   * @return {Point} this
   */
  public shift(x: number, y: number): Point {
    this.x += x;
    this.y += y;
    return this;
  }

  /**
   * Scales the point by some amount.
   * @param {Number} xOrBoth X scalar
   * @param {Number} y Y scalar
   * @return {Point} this
   */
  public scale(xOrBoth: number, y?: number): Point {
    if (y === undefined) { y = xOrBoth; }
    this.x *= xOrBoth;
    this.y *= y;
    return this;
  }

  /**
   * Normalizes this point.
   * Effectively makes this the length of 1 unit but keeps the direction unchanged.
   * (Extremely small rounding errors will most likey occur, due to the way Number works)
   *
   * __Note:  To get a new point with the normal, use ".clone().normalize()" instead__
   * @return {Point} this
   */
  public normalize(): Point {
    var dir = Math.atan2(this.y, this.x);
    this.x = Math.cos(dir);
    this.y = Math.sin(dir);
    return this;
  }

  /**
   * Rotates the point around the origin.
   * (Extremely small rounding errors will most likey occur, due to the way Number works)
   * @param {Number} rotation Amount to rotate (Clockwise in radians)
   * @return {Point} this
   */
  public rotate(rotation: number): Point {
    var len = this.length();
    var dir = Math.atan2(this.y, this.x) + rotation;
    this.x = Math.cos(dir) * len;
    this.y = Math.sin(dir) * len;
    return this;
  }

  /**
   * Copy the X and Y from another point.
   * @param {Point} source Point to copy X & Y from
   * @return {Point} this
   */
  public copy(source: IPoint): Point {
    this.x = source.x;
    this.y = source.y;
    return this;
  }

  /**
   * Creates a copy of this (a new point with the same X and Y values).
   * @return {Point} Copy of this
   */
  public clone(): Point {
    return new Point(this.x, this.y);
  }

  /**
   * Creates a string representation of this Point
   * @returns String of this point
   */
  toString(): string {
    return 'x: '+this.x+', y: '+this.y;
  }
}

/**
 * Any object that can be used as a Point
 */
export interface IPoint {
  x: number;
  y: number;
}

