import { Point } from './Point';

/**
 * Arbitrary collection of Points
 * (More or less an array wrapper with helpful methods)
 */
export class PointCloud {
  /**
   * Points in this point cloud
   */
  public points: Point[] = [];

  constructor(points?: Point[]) {
    if (points) {
      for (let point of points) {
        if (point) { this.points.push(point); }
      }
    }
  }

  /**
   * Calculate the average of all points
   */
  public average(): Point {
    let sum: Point = new Point();
    for (let point of this.points) {
      sum.shift(point.x, point.y);
    }
    return sum.scale(1 / this.points.length);
  }

  /**
   * Calculate the center of this point could
   * (The average of the outer-most points of this cloud)
   */
  public center(): Point {
    let minX: number, maxX: number, minY: number, maxY: number;
    minX = minY = Number.POSITIVE_INFINITY;
    maxX = maxY = Number.NEGATIVE_INFINITY;
    for (let point of this.points) {
      if      (point.x > maxX) { maxX = point.x; }
      else if (point.x < minX) { minX = point.x; }
      if      (point.y > maxY) { maxY = point.y; }
      else if (point.y < minY) { minY = point.y; }
    }
    return new Point((minX + maxX) * 0.5, (minY + maxY) * 0.5);
  }

  /**
   * Shift all points in this could
   * @param x X difference
   * @param y Y difference
   * @return {PointCloud} This (enables chaining)
   */
  public shift(x: number, y: number): PointCloud {
    for (let point of this.points) {
      point.shift(x, y);
    }
    return this;
  }
  
  /**
   * Scale all of the points in this cloud
   * @param {Number} xOrBoth X scalar
   * @param {Number} y Y scalar
   * @return {Point} this
   */
  public scale(xOrBoth: number, y?: number): PointCloud {
    if (y === undefined) { y = xOrBoth; }
    for (let point of this.points) {
      point.scale(xOrBoth, y);
    }
    return this;
  }

  /**
   * Create a clone of this point cloud
   * @param clonePoints If the clone should have points of it own, instead of using the same points as this
   */
  public clone(clonePoints: boolean = true): PointCloud {
    if (clonePoints) {
      let points: Point[] = [];
      for (let point of this.points) {
        points.push(point.clone());
      }
      return new PointCloud(points);
    } else {
      return new PointCloud(this.points);
    }
  }
}
