/**
 * 2D Rectangle
 * @class Rectangle
 * @property {Number} left Position of the rectangles left side
 * @property {Number} top Position of the rectangles top
 * @property {Number} right Position of the rectangles right side
 * @property {Number} bottom Position of the rectangles bottom
 */
export class Rectangle implements IRectangle {
  public left: number;
  public top: number;
  public right: number;
  public bottom: number;

  /**
   * Create a rectangle
   * @param left Position of the rectangles left side
   * @param top Position of the rectangles top
   * @param right Position of the rectangles right side
   * @param bottom Position of the rectangles bottom
   */
  constructor(left: number = 0, top: number = 0, right: number = 0, bottom: number = 0) {
    this.left   = left;
    this.top    = top;
    this.right  = right;
    this.bottom = bottom;
  }

  /**
   * Shift the position of this rectangle by some amount
   * @param horizontal Amount to shift horizontally (left/right)
   * @param vertical Amount to shift vertically (up/down)
   */
  public shift(horizontal: number, vertical: number): Rectangle {
    this.left   += horizontal;
    this.right  += horizontal;
    this.top    += vertical;
    this.bottom += vertical;
    return this;
  }

  /**
   * @TODO
   */
  public moveTo(left: number, top: number): Rectangle {
    // Move "right" relative to "left", so that the height is the same
    this.right = this.right - this.left + left;
    this.left = left;
    // Move "bottom" relative to "top", so that the height is the same
    this.bottom = this.bottom - this.top + top;
    this.top = top;
    return this;
  }

  /**
   * Get the width of the rectangle
   * @returns Width of the rectangle
   */
  public getWidth(): number {
    return this.right - this.left;
  }

  /**
   * Get the height of the rectangle
   * @returns Height of the rectangle
   */
  public getHeight(): number {
    return this.bottom - this.top;
  }

  /**
   * Get the area of the rectangle
   * @returns Area of the rectangle
   */
  public getArea(): number {
    return (this.right - this.left) * (this.bottom - this.top);
  }

  /**
   * Get the intersecting area of this rectangle and another
   * (returns null if no intersection was made)
   * @param other Rectangle to check intersection with
   * @return A new rectangle of the intersecting area (null if no intersection was made)
   */
  public intersection(other: Rectangle): Rectangle|null {
    var rect: Rectangle = new Rectangle(Math.max(this.left,   other.left),
                                       Math.min(this.top,    other.top),
                                       Math.max(this.right,  other.right),
                                       Math.min(this.bottom, other.bottom));
    if (rect.getArea() > 0) { return rect; }
    return null;
  }

  /**
   * Check if this rectangle intersects with anther
   * @param other Rectangle to check intersection with
   * @returns If the rectangles intersected
   */
  public isIntersecting(other: IRectangle): boolean {
    if (this.left   > other.right  ||
        this.top    > other.bottom ||
        this.right  < other.left   ||
        this.bottom < other.top) {
      return false;
    }
    return true;
  }

  /**
   * Creates a copy of this rectangle
   * @returns Copy of this rectangle
   */
  public clone(): Rectangle {
    return new Rectangle(this.left, this.right, this.top, this.bottom);
  }
}

export interface IRectangle {
  left: number;
  top: number;
  right: number;
  bottom: number;
}
